"""
Crewai Enterprise Tools
"""

import os
import typing as t
from crewai.tools import BaseTool
from crewai_tools.adapters.enterprise_adapter import EnterpriseActionKitToolAdapter


def CrewaiEnterpriseTools(
    enterprise_token: t.Optional[str] = None,
    actions_list: t.Optional[t.List[str]] = None,
    enterprise_action_kit_project_id: t.Optional[str] = None,
    enterprise_action_kit_project_url: t.Optional[str] = None,
) -> t.List[BaseTool]:
    """Factory function that returns crewai enterprise tools.

    Args:
        enterprise_token: The token for accessing enterprise actions.
                         If not provided, will try to use CREWAI_ENTEPRISE_TOOLS_TOKEN env var.
        actions_list: Optional list of specific tool names to include.
                   If provided, only tools with these names will be returned.

    Returns:
        A list of BaseTool instances for enterprise actions
    """
    if enterprise_token is None:
        enterprise_token = os.environ.get("CREWAI_ENTEPRISE_TOOLS_TOKEN")
        if enterprise_token is None:
            raise ValueError(
                "No enterprise token provided. Please provide a token or set the CREWAI_ENTEPRISE_TOOLS_TOKEN environment variable."
            )

    adapter_kwargs = {"enterprise_action_token": enterprise_token}

    if enterprise_action_kit_project_id is not None:
        adapter_kwargs["enterprise_action_kit_project_id"] = (
            enterprise_action_kit_project_id
        )
    if enterprise_action_kit_project_url is not None:
        adapter_kwargs["enterprise_action_kit_project_url"] = (
            enterprise_action_kit_project_url
        )

    adapter = EnterpriseActionKitToolAdapter(**adapter_kwargs)
    all_tools = adapter.tools()

    if actions_list is None:
        return all_tools

    # Filter tools based on the provided list
    return [tool for tool in all_tools if tool.name in actions_list]



import json
from typing import List, Optional

import pytest
from pydantic import BaseModel, Field
from unittest import mock

from generate_tool_specs import ToolSpecExtractor
from crewai.tools.base_tool import EnvVar

class MockToolSchema(BaseModel):
    query: str = Field(..., description="The query parameter")
    count: int = Field(5, description="Number of results to return")
    filters: Optional[List[str]] = Field(None, description="Optional filters to apply")


class MockTool:
    name = "Mock Search Tool"
    description = "A tool that mocks search functionality"
    args_schema = MockToolSchema

@pytest.fixture
def extractor():
    ext = ToolSpecExtractor()
    MockTool.__pydantic_core_schema__ = create_mock_schema(MockTool)
    MockTool.args_schema.__pydantic_core_schema__ = create_mock_schema_args(MockTool.args_schema)
    return ext


def create_mock_schema(cls):
    return {
        "type": "model",
        "cls": cls,
        "schema": {
            "type": "model-fields",
            "fields": {
                "name": {"type": "model-field", "schema": {"type": "default", "schema": {"type": "str"}, "default": cls.name}, "metadata": {}},
                "description": {"type": "model-field", "schema": {"type": "default", "schema": {"type": "str"}, "default": cls.description}, "metadata": {}},
                "args_schema": {"type": "model-field", "schema": {"type": "default", "schema": {"type": "is-subclass", "cls": BaseModel}, "default": cls.args_schema}, "metadata": {}},
                "env_vars": {
                    "type": "model-field", "schema": {"type": "default", "schema": {"type": "list", "items_schema": {"type": "model", "cls": "INSPECT CLASS", "schema": {"type": "model-fields", "fields": {"name": {"type": "model-field", "schema": {"type": "str"}, "metadata": {}}, "description": {"type": "model-field", "schema": {"type": "str"}, "metadata": {}}, "required": {"type": "model-field", "schema": {"type": "default", "schema": {"type": "bool"}, "default": True}, "metadata": {}}, "default": {"type": "model-field", "schema": {"type": "default", "schema": {"type": "nullable", "schema": {"type": "str"}}, "default": None}, "metadata": {}},}, "model_name": "EnvVar", "computed_fields": []}, "custom_init": False, "root_model": False, "config": {"title": "EnvVar"}, "ref": "crewai.tools.base_tool.EnvVar:4593650640", "metadata": {"pydantic_js_functions": ["INSPECT __get_pydantic_json_schema__"]}}}, "default": [EnvVar(name='SERPER_API_KEY', description='API key for Serper', required=True, default=None), EnvVar(name='API_RATE_LIMIT', description='API rate limit', required=False, default="100")]}, "metadata": {}
                }
            },
            "model_name": cls.__name__
        }
    }


def create_mock_schema_args(cls):
    return {
        "type": "model",
        "cls": cls,
        "schema": {
            "type": "model-fields",
            "fields": {
                "query": {"type": "model-field", "schema": {"type": "default", "schema": {"type": "str"}, "default": "The query parameter"}},
                "count": {"type": "model-field", "schema": {"type": "default", "schema": {"type": "int"}, "default": 5}, "metadata": {"pydantic_js_updates": {"description": "Number of results to return"}}},
                "filters": {"type": "model-field", "schema": {"type": "nullable", "schema": {"type": "list", "items_schema": {"type": "str"}}}}
            },
            "model_name": cls.__name__
        }
    }


def test_unwrap_schema(extractor):
    nested_schema = {
        "type": "function-after",
        "schema": {"type": "default", "schema": {"type": "str", "value": "test"}}
    }
    result = extractor._unwrap_schema(nested_schema)
    assert result["type"] == "str"
    assert result["value"] == "test"


@pytest.mark.parametrize(
    "field, fallback, expected",
    [
        ({"schema": {"default": "test_value"}}, None, "test_value"),
        ({}, "fallback_value", "fallback_value"),
        ({"schema": {"default": 123}}, "fallback_value", "fallback_value")
    ]
)
def test_extract_field_default(extractor, field, fallback, expected):
    result = extractor._extract_field_default(field, fallback=fallback)
    assert result == expected


@pytest.mark.parametrize(
    "schema, expected",
    [
        ({"type": "str"}, "str"),
        ({"type": "list", "items_schema": {"type": "str"}}, "list[str]"),
        ({"type": "dict", "keys_schema": {"type": "str"}, "values_schema": {"type": "int"}}, "dict[str, int]"),
        ({"type": "union", "choices": [{"type": "str"}, {"type": "int"}]}, "union[str, int]"),
        ({"type": "custom_type"}, "custom_type"),
        ({}, "unknown"),
    ]
)
def test_schema_type_to_str(extractor, schema, expected):
    assert extractor._schema_type_to_str(schema) == expected


@pytest.mark.parametrize(
    "info, expected_type",
    [
        ({"schema": {"type": "str"}}, "str"),
        ({"schema": {"type": "nullable", "schema": {"type": "int"}}}, "int"),
        ({"schema": {"type": "default", "schema": {"type": "list", "items_schema": {"type": "str"}}}}, "list[str]"),
    ]
)
def test_extract_param_type(extractor, info, expected_type):
    assert extractor._extract_param_type(info) == expected_type


def test_extract_tool_info(extractor):
    with mock.patch("generate_tool_specs.dir", return_value=["MockTool"]), \
         mock.patch("generate_tool_specs.getattr", return_value=MockTool):
        extractor.extract_all_tools()

        assert len(extractor.tools_spec) == 1
        tool_info = extractor.tools_spec[0]

        assert tool_info["name"] == "MockTool"
        assert tool_info["humanized_name"] == "Mock Search Tool"
        assert tool_info["description"] == "A tool that mocks search functionality"

        assert len(tool_info["env_vars"]) == 2
        api_key_var, rate_limit_var = tool_info["env_vars"]

        assert api_key_var["name"] == "SERPER_API_KEY"
        assert api_key_var["description"] == "API key for Serper"
        assert api_key_var["required"] == True
        assert api_key_var["default"] == None

        assert rate_limit_var["name"] == "API_RATE_LIMIT"
        assert rate_limit_var["description"] == "API rate limit"
        assert rate_limit_var["required"] == False
        assert rate_limit_var["default"] == "100"

        assert len(tool_info["run_params"]) == 3

        params = {p["name"]: p for p in tool_info["run_params"]}
        assert params["query"]["description"] == "The query parameter"
        assert params["query"]["type"] == "str"

        assert params["count"]["description"] == "Number of results to return"
        assert params["count"]["type"] == "int"

        assert params["filters"]["description"] == ""
        assert params["filters"]["type"] == "list[str]"


def test_save_to_json(extractor, tmp_path):
    extractor.tools_spec = [{
        "name": "TestTool",
        "humanized_name": "Test Tool",
        "description": "A test tool",
        "run_params": [
            {"name": "param1", "description": "Test parameter", "type": "str"}
        ]
    }]

    file_path = tmp_path / "output.json"
    extractor.save_to_json(str(file_path))

    assert file_path.exists()

    with open(file_path, "r") as f:
        data = json.load(f)

    assert "tools" in data
    assert len(data["tools"]) == 1
    assert data["tools"][0]["humanized_name"] == "Test Tool"
    assert data["tools"][0]["run_params"][0]["name"] == "param1"


@pytest.mark.integration
def test_full_extraction_process():
    extractor = ToolSpecExtractor()
    specs = extractor.extract_all_tools()

    assert len(specs) > 0

    for tool in specs:
        assert "name" in tool
        assert "humanized_name" in tool and tool["humanized_name"]
        assert "description" in tool
        assert isinstance(tool["run_params"], list)
        for param in tool["run_params"]:
            assert "name" in param and param["name"]


#!/usr/bin/env python3

import inspect
import json
from pathlib import Path
from typing import Any, Dict, List, Optional, Type

from crewai_tools import tools
from crewai.tools.base_tool import EnvVar


class ToolSpecExtractor:
    def __init__(self) -> None:
        self.tools_spec: List[Dict[str, Any]] = []
        self.processed_tools: set[str] = set()

    def extract_all_tools(self) -> List[Dict[str, Any]]:
        for name in dir(tools):
            if name.endswith("Tool") and name not in self.processed_tools:
                obj = getattr(tools, name, None)
                if inspect.isclass(obj):
                    self.extract_tool_info(obj)
                    self.processed_tools.add(name)
        return self.tools_spec

    def extract_tool_info(self, tool_class: Type) -> None:
        try:
            core_schema = tool_class.__pydantic_core_schema__
            if not core_schema:
                return

            schema = self._unwrap_schema(core_schema)
            fields = schema.get("schema", {}).get("fields", {})
            tool_info = {
                "name": tool_class.__name__,
                "humanized_name": self._extract_field_default(fields.get("name"), fallback=tool_class.__name__),
                "description": self._extract_field_default(fields.get("description")).strip(),
                "run_params": self._extract_params(fields.get("args_schema")),
                "env_vars": self._extract_env_vars(fields.get("env_vars")),
            }

            self.tools_spec.append(tool_info)

        except Exception as e:
            print(f"Error extracting {tool_class.__name__}: {e}")

    def _unwrap_schema(self, schema: Dict) -> Dict:
        while schema.get("type") in {"function-after", "default"} and "schema" in schema:
            schema = schema["schema"]
        return schema

    def _extract_field_default(self, field: Optional[Dict], fallback: str = "") -> str:
        if not field:
            return fallback
            
        schema = field.get("schema", {})
        default = schema.get("default")
        return default if isinstance(default, str) else fallback

    def _extract_params(self, args_schema_field: Optional[Dict]) -> List[Dict[str, str]]:
        if not args_schema_field:
            return []

        args_schema_class = args_schema_field.get("schema", {}).get("default")
        if not (inspect.isclass(args_schema_class) and hasattr(args_schema_class, "__pydantic_core_schema__")):
            return []

        try:
            core_schema = args_schema_class.__pydantic_core_schema__
            schema = self._unwrap_schema(core_schema)
            fields = schema.get("schema", {}).get("fields", {})

            params = []
            for name, info in fields.items():
                _type = self._extract_param_type(info)
                if _type == "union":
                    breakpoint()
                param = {
                    "name": name,
                    "description": self._extract_field_default(info)
                        or self._extract_field_description_from_metadata(info),
                    "type": _type,
                }
                params.append(param)

            return params

        except Exception as e:
            print(f"Error extracting params from {args_schema_class}: {e}")
            return []

    def _extract_env_vars(self, env_vars_field: Optional[Dict]) -> List[Dict[str, str]]:
        if not env_vars_field:
            return []

        env_vars = []
        for env_var in env_vars_field.get("schema", {}).get("default", []):
            if isinstance(env_var, EnvVar):
                env_vars.append({
                    "name": env_var.name,
                    "description": env_var.description,
                    "required": env_var.required,
                    "default": env_var.default,
                })
        return env_vars
    
    def _extract_field_description_from_metadata(self, field: Dict) -> str:
        if metadata := field.get("metadata"):
            return metadata.get("pydantic_js_updates", {}).get("description", "")
        return ""

    def _extract_param_type(self, info: Dict) -> Optional[str]:
        schema = info.get("schema", {})
        schema = self._unwrap_schema(schema)

        if schema.get("type") == "nullable":
            inner = schema.get("schema", {})
            return self._schema_type_to_str(inner)

        return self._schema_type_to_str(schema)

    def _schema_type_to_str(self, schema: Dict) -> str:
        schema_type = schema.get("type", "")

        if schema_type == "list" and "items_schema" in schema:
            item_type = self._schema_type_to_str(schema["items_schema"])
            return f"list[{item_type}]"
        
        if schema_type == "union" and "choices" in schema:
            choices = schema["choices"]
            item_types = [self._schema_type_to_str(choice) for choice in choices]
            return f"union[{', '.join(item_types)}]"

        if schema_type == "dict" and "keys_schema" in schema and "values_schema" in schema:
            key_type = self._schema_type_to_str(schema["keys_schema"])
            value_type = self._schema_type_to_str(schema["values_schema"])
            return f"dict[{key_type}, {value_type}]"

        return {
            "str": "str",
            "int": "int",
            "float": "float",
            "bool": "bool",
            "list": "list",
            "dict": "dict",
            "any": "any",
        }.get(schema_type, schema_type or "unknown")

    def save_to_json(self, output_path: str) -> None:
        with open(output_path, "w", encoding="utf-8") as f:
            json.dump({"tools": self.tools_spec}, f, indent=2, sort_keys=True)
        print(f"Saved tool specs to {output_path}")


if __name__ == "__main__":
    output_file = Path(__file__).parent / "tool.specs.json"
    extractor = ToolSpecExtractor()

    specs = extractor.extract_all_tools()
    extractor.save_to_json(str(output_file))

    print(f"Extracted {len(specs)} tool classes.")